@@ -691,8 +691,10 @@ message_get_header (GMimeObject *object, const char *header)
 	if (!strcasecmp ("MIME-Version", header))
 		return "1.0";
 	
-	if (strncasecmp ("Content-", header, 8))
+	if (strncasecmp ("Content-", header, 8) != 0)
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
+	else if (((GMimeMessage *) object)->mime_part)
+		return g_mime_object_get_header (((GMimeMessage *) object)->mime_part, header);
 	else
 		return NULL;
 }
@@ -1501,7 +1503,7 @@ g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gbool
 	
 	type = g_mime_object_get_content_type (message->mime_part);
 	if (GMIME_IS_MULTIPART (message->mime_part)) {
-		/* lets see if we can find a body in the multipart */
+		/* let's see if we can find a body in the multipart */
 		multipart = GMIME_MULTIPART (message->mime_part);
 		if (g_mime_content_type_is_type (type, "multipart", "alternative"))
 			mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
