@@ -659,7 +659,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	}
 	
 	d(printf ("\n"));
-		
+	
 	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
@@ -736,7 +736,12 @@ g_mime_utils_header_decode_date (const char *in, int *saveoffset)
 	struct _date_token *token, *tokens;
 	time_t date;
 	
-	tokens = datetok (in);
+	if (!(tokens = datetok (in))) {
+		if (saveoffset)
+			*saveoffset = 0;
+		
+		return (time_t) 0;
+	}
 	
 	date = parse_rfc822_date (tokens, saveoffset);
 	if (!date)
@@ -976,6 +981,21 @@ g_mime_references_clear (GMimeReferences **refs)
 }
 
 
+/**
+ * g_mime_references_next:
+ * @ref: a GMimeReferences list
+ *
+ * Advances to the next reference node in the GMimeReferences list.
+ *
+ * Returns the next reference node in the GMimeReferences list.
+ **/
+GMimeReferences *
+g_mime_references_next (const GMimeReferences *ref)
+{
+	return ref ? ref->next : NULL;
+}
+
+
 static gboolean
 is_rfc2047_token (const char *inptr, size_t len)
 {
